#!/usr/bin/python
# -*- coding: utf-8 -*-

import os, sys
import copy
import time

import technoap.sitcp as sitcp
import technoap.device.apu101 as apu101

	
#device IP address
DEST_IPADDR = "192.168.10.128"

##########################################################
def main(argv):

	print ("start acquisition...")

	try:
		#create 'RBCP' instance
		rbcp = sitcp.rbcp.RBCP(True)
		rbcp.setdefaultdest(DEST_IPADDR)

		#for measurement device, instance
		config = apu101.config.Config(rbcp)
		status = apu101.status.Status(rbcp)
		datatx = apu101.fastdatatx.FastDataTX()
		datatx.setDefaultRBCP(rbcp)
		isConnected = datatx.connectDevice(100, DEST_IPADDR)	#device connecting
		
		########################################################
		if isConnected == True:
			config.stopAcquisition()	#Issue measurement stop instruction just in case
			init_config(config)			#parameter setting
			acquisition(config, status, datatx, 5.0)	#5.0sec
			#acquisition(config, status, datatx, 15.0)	#15.0sec
		else:
			print ("device connect error...")
		########################################################

	except Exception as ex:
		print (ex)

	print ("acquisition complete!!!")

##########################################################
def acquisition(config, status, datatx, acq_sec):

	dataHist = None

	try:
		config.clearAcquisitionData()	#acquisition data clear
		config.setAcquisitionMode(apu101.constant.REG_MOD_HISTOGRAM)		#histogram
		config.setAcquisitionTimeMode(apu101.constant.REG_MMD_REALTIME)		#real time
		config.setAcquisitionTime(acq_sec)	#setting acquisition time

		config.startAcquisition()	#acquisition start!!!

		runAcq = True

		while runAcq == True:
			#reading status info
			status.readStatusAll()
			elapseSeconds = status.getElapsedSeconds(False)		#elapsed time(sec)

			#### acq finish #####
			if elapseSeconds >= acq_sec:
				status.readStatusAll()	#Re-read at the end of acquisition
				runAcq = False			#end acquisition

			#reading histogram data
			dataHist = datatx.readHistogram(1, 4096)		#CH1

			#########################################
			print ("\n########## acquisition time: %.1fs / %.1fs ##########" % (elapseSeconds, acq_sec))
			print ("input count = %d, input count rate = %d." % (status.mStatus["ICT",], status.mStatus["ICR",]))
			print ("throughput count = %d, throughput count rate = %d." % (status.mStatus["TCT",], status.mStatus["TCR",]))

			time.sleep(0.5)		#wait

		#####histogram save#####
		with open('apu101.csv', 'w') as csv:
			csv.writelines("[header]\n")
			csv.writelines("real time(s),%f\n" % (elapseSeconds))
			csv.writelines("input rate(cps),%d\n" % (status.mStatus["ICR",]))
			csv.writelines("throughput rate(cps),%d\n" % (status.mStatus["TCR",]))

			csv.writelines("[data]\n")
			csv.writelines("#bin,counts\n")

			for i in range(len(dataHist)):
				textline = "%d,%d\n" % (i, dataHist[i])
				csv.writelines(textline)
		########################
	except Exception as ex:
		print (ex)
	
	config.stopAcquisition()

	return dataHist

##########################################################
def init_config(config):

	config.clearStatus()	#config clear

	########## common ##########
	config.mStatus["CLS",] = apu101.constant.REG_CLS_INTERNAL	#internal
	config.mStatus["DAC",] = apu101.constant.REG_DAC_PREAMP
	config.mStatus["RCH",] = [
		apu101.constant.REG_RCH_CH1,	#1st: CH1
		apu101.constant.REG_RCH_NONE,	#2nd: unuse
		apu101.constant.REG_RCH_NONE,	#3rd: unuse
		apu101.constant.REG_RCH_NONE	#4th: unuse
	]
	config.mStatus["RRG",] = [
		0, 8191,		#1st: start:1024ch, end:2048ch
		0, 0,			#2nd: No setting(=0) is required because it is not used in the RCH register.
		0, 0,			#3rd: No setting(=0) is required because it is not used in the RCH register. 
		0, 0			#4th: No setting(=0) is required because it is not used in the RCH register.
	]

	########## CH ##########
	config.mStatus["ACG",] = apu101.constant.REG_ACG_X002	#2 times
	config.mStatus["ADG",] = apu101.constant.REG_ADG_4K		#4096 bin
	config.mStatus["FFD",] = apu101.constant.REG_FFD_20
	config.mStatus["FFI",] = apu101.constant.REG_FFI_20
	sfr = 6000	#6000nsec
	config.mStatus["SFR",] = config.toDigitSFR(sfr)
	config.mStatus["SFP",] = config.toDigitSFP(sfr, 700)	#sfp = 700 nsec
	config.mStatus["FPZ",] =    0	#    0 ch, fast pole zero
	config.mStatus["SPZ",] =  680	#  690 ch, slow pole zero
	config.mStatus["FTH",] =   30	#   30 ch, fast trigger threshold
	config.mStatus["LLD",] =   30	#   30 ch, energy lower level discriminator
	config.mStatus["ULD",] = 8191	# 8191 ch, energy upper level discriminator
	config.mStatus["STH",] =   30	#   30 ch, slow trigger threshold
	config.mStatus["PUR",] = apu101.constant.REG_PUR_OFF	#off
	config.mStatus["POL",] = apu101.constant.REG_POL_POS	#positive
	config.mStatus["DCG",] = apu101.constant.REG_DCG_X32	#32 times
	config.mStatus["DFG",] = config.toDigitDFG(0.5)
	config.mStatus["TMS",] = apu101.constant.REG_TMS_CFD	#constant fraction discriminator timing
	config.mStatus["CFF",] = config.toDigitCFF(apu101.constant.REG_CFF_125)	#CFD function
	config.mStatus["CFD",] = apu101.constant.REG_CFD_10NS	#CFD delay
	config.mStatus["IHW",] = config.toDigitIHW(10)			#inihibit width: 100 microseconds
	config.mStatus["DIF",] = apu101.constant.REG_DIF_68US	#coupling: 6.8 usec
	config.mStatus["PZD",] =  128	# 128 ch
	config.mStatus["FGD",] = config.toDigitFGD(255)			#analog fine gain
	config.mStatus["BTS",] = apu101.constant.REG_BTS_DEFAULT
	config.mStatus["BRS",] = apu101.constant.REG_BRS_GE		#germanium(resistance feedback type preamplifier)

	#Reflect settings to device
	result = config.writeStatusAll()
	time.sleep(0.5)		#Be sure to wait about 500 msec after writing all parameters.

	if result == False:
		raise Exception('config write error...')


##########################################################
if __name__ == "__main__":
	main(sys.argv)


