#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import math
import time

from . import constant
from . import register
from ..common import registerRW
from ...num import bitconverter

#target register list
TARGET_REGISTER_LIST = [
	"HSW",
	"HSP",
	"HSJ",
	"HVD"
]

#device specific value
TARGET_SPECIFIC_REG_LIST = [
	"PDO",	#POS: setting offset
	"PDF",	#POS: setting fine gain
	"NDO",	#NEG: setting offset
	"NDF",	#NEG: setting value fine gain
	"PVO",	#POS: Voltage monitor value offset
	"PVF",	#POS: Voltage monitor value fine gain
	"NVO",	#NEG: Voltage monitor value offset
	"NVF",	#NEG: Voltage monitor value fine gain
	"PIO",	#POS: Current monitor value offset
	"PIF",	#POS: Current monitor fine gain
	"NIO",	#NEG: Current monitor offset
	"NIF"	#NEG: Current monitor fine gain
]

################################################
POLARITY_HIZ = 0x00
POLARITY_NEG = 0x02
POLARITY_POS = 0x03


class ConfigHV(registerRW.RegisterRW):
	
	###############################################
	def __init__(self, rbcp = None):
		self.__initializing(True)
		super(ConfigHV, self).__init__(rbcp)

		self.mRegisterAttribute = register.Register()
		self.createRegisterInfo(TARGET_REGISTER_LIST, self.mRegisterAttribute)
	
	###############################################
	def __initializing(self, runParent = False):
		if runParent == True:
			super(ConfigHV, self)._RegisterRW__initializing()

	###############################################
	def outputVolatageON(self, ipaddr = None, port = None):
		return self.writeRegisterByName("HEN", constant.REG_HEN_ON, ipaddr, port)

	###############################################
	def outputVolatageOFF(self, ipaddr = None, port = None):
		return self.writeRegisterByName("HEN", constant.REG_HEN_OFF, ipaddr, port)

	###############################################
	def setPolarity(self, pol = POLARITY_HIZ, ipaddr = None, port = None):

		result = True

		try:
			attb_hpc = self.getAttribute("HPC")
			attb_hpl = self.getAttribute("HPL")

			########## writing ##########
			list_attrb = [attb_hpc, attb_hpl, attb_hpc]
			list_value = [1, pol, 0]

			for i in range(len(list_attrb)):
				attrb = list_attrb[i]
				value = list_value[i]
				resultW = self.mRBCP.writeNbytes(attrb.mAddress, value, attrb.mBytes, ipaddr, port)
				time.sleep(0.1)

				if resultW != True:
					result = False
					break

			########## reading ##########
			if result == True:
				list_attrb = [attb_hpl, attb_hpl]
				list_value = []

				for i in range(len(list_attrb)):
					attrb = list_attrb[i]
					value = self.mRBCP.readNbytes(attrb.mAddress, attrb.mBytes, ipaddr, port)
					list_value.append(value)
					time.sleep(0.1)

					if value is None:
						result = False
						break

		except Exception as ex:
			self.mLastException = ex
			result = False
		
		return result

	###############################################
	"""
	#Pass parameter(reg_list) in dictionary format.
	#Contact TechnoAP for the value as it is a device-specific value.
	#An example is shown below.
	reglist = {
		"PDO": N,	#POS: setting offset
		"PDF": N,	#POS: setting fine gain
		"NDO": N,	#NEG: setting offset
		"NDF": N,	#NEG: setting value fine gain
		"PVO": N,	#POS: Voltage monitor value offset
		"PVF": N,	#POS: Voltage monitor value fine gain
		"NVO": N,	#NEG: Voltage monitor value offset
		"NVF": N,	#NEG: Voltage monitor value fine gain
		"PIO": N,	#POS: Current monitor value offset
		"PIF": N,	#POS: Current monitor fine gain
		"NIO": N,	#NEG: Current monitor offset
		"NIF": N	#NEG: Current monitor fine gain
	}
	"""
	def setDeviceSpecificConstants(self, reg_list, ipaddr = None, port = None):
		
		result = True

		try:
			for i in range(len(TARGET_SPECIFIC_REG_LIST)):
				reg_name = TARGET_SPECIFIC_REG_LIST[i]
				attrb = self.getAttribute(reg_name)
				value = reg_list[reg_name]
				resultW = self.mRBCP.writeNbytes(attrb.mAddress, value, attrb.mBytes, ipaddr, port)

				if resultW != True:
					result = False
					break
		except Exception as ex:
			self.mLastException = ex
			result = False

		return result

	###############################################
	@staticmethod
	def toDigitHSW(Vmin, implementedmodule):
		if float(implementedmodule) == constant.HVMODULE_IMPLEMENTED_1KV:	#1000V module
			digit = float(constant.REG_HSW_COEFFICIENT_1KV) / float(Vmin)
			digit = bitconverter.inRangeAndCoarce(digit, constant.REG_HSW_MIND, constant.REG_HSW_MAXD)
		elif float(implementedmodule) == constant.HVMODULE_IMPLEMENTED_4KV:	#4000V module
			digit = float(constant.REG_HSW_COEFFICIENT_4KV) / float(Vmin)
			digit = bitconverter.inRangeAndCoarce(digit, constant.REG_HSW_MIND, constant.REG_HSW_MAXD)
		else:
			digit = None

		return int(digit)
		
	###############################################
	@staticmethod
	def toDigitHSJ(volts):
		digit0  = float(volts) * constant.REG_HSJ_COEFFICIENT_A
		digit0 += constant.REG_HSJ_COEFFICIENT_B
		digit   = bitconverter.inRangeAndCoarce(int(digit0), constant.REG_HSJ_MIND, constant.REG_HSJ_MAXD)
		return digit


	###############################################
	@staticmethod
	def toDigitHVD(volts, maxoutput):
		if float(maxoutput) == constant.HVMODULE_IMPLEMENTED_1KV:	#1000V module
			digit0 = (float(math.fabs(volts)) / constant.REG_HVD_COEFFICIENT_1KV) * constant.REG_HVD_COEFFICIENT_B
		elif float(maxoutput) == constant.HVMODULE_IMPLEMENTED_4KV:	#4000V module
			digit0 = (float(math.fabs(volts)) / constant.REG_HVD_COEFFICIENT_4KV) * constant.REG_HVD_COEFFICIENT_B
		else:
			digit0 = None
		digit  =  bitconverter.inRangeAndCoarce(int(digit0), constant.REG_HVD_MIND, constant.REG_HVD_MAXD)
		return digit
			

	



