#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import struct

from . import constant
from ..num import byteorder, bitconverter


BYTEORDER_HOST    = 0x00	#host byte order
BYTEORDER_NETWORK = 0x01	#network byte order
BYTEORDER_LITTLE  = 0x10	#little endian
BYTEORDER_BIG     = 0x11	#big endian


class RBCPFormat(object):
	
	##############################################################
	def __init__(self):
		self.__initializing()

	##############################################################
	def __initializing(self):
		self.mSequenceTX = 0
		self.clear()
	
	##############################################################
	def clear(self):
		self.mVersion = 0
		self.mType = 0
		self.mCmd = 0
		self.mFlag = 0
		self.mID = 0
		self.mLength = 0
		self.mAddress = 0
		self.mData = None
		self.mDataFull = None

		self.mEnabled = False
		self.mLastException = None

	##############################################################
	def incrementID(self):
		self.mSequenceTX += 1
		if not (0 <= self.mSequenceTX <= 255):
			self.mSequenceTX = 0
		return self.mSequenceTX

	##############################################################
	def generateReadCommand(self, address, nbytes):
		self.mDataFull = self.__generateCommand(address, nbytes)
		return self.mDataFull
	
	##############################################################
	def generateWriteCommand(self, address, data):
		nbytes = len(data)
		self.mDataFull = self.__generateCommand(address, nbytes, data)
		return self.mDataFull

	##############################################################
	def __generateCommand(self, address, nbytes, data = None):
		
		text0 = None
		header = None

		id = self.incrementID()

		try:
			if data is None:	#read command
				header = struct.pack("!BBBBI", constant.RBCP_VER_TYPE, constant.RBCP_CMD_READING, id, nbytes, address)
				#v1, v2, v3, v4, v5 = struct.unpack("!BBBBI", header)
				text = header
			else:
				header = struct.pack("!BBBBI", constant.RBCP_VER_TYPE, constant.RBCP_CMD_WRITING, id, nbytes, address)
				text = header + data

			text0 = bytearray(text)
			self.restoreCommand(text0)
		except Exception as ex:
			self.mLastException = ex
			raise(ex)

		return text0

	##############################################################
	def restoreCommand(self, data, order = BYTEORDER_NETWORK):

		result = False
		self.mEnabled = False

		works = self.mLastException
		self.clear()
		self.mLastException = works

		try:
			nbytes = len(data)

			if nbytes >= 8:
				self.mVersion = data[0] & 0xf0
				self.mType    = data[0] & 0x0f
				self.mCmd     = data[1] & 0xf0
				self.mFlag    = data[1] & 0x0f
				self.mID      = data[2]
				self.mLength  = data[3]
				self.mAddress = bitconverter.toIntU32(data, 4)

				if order == BYTEORDER_NETWORK or (order == BYTEORDER_BIG and sys.byteorder == 'little'):
					self.mAddress = byteorder.ntohl(self.mAddress)

				if nbytes > 8:
					self.mData = bitconverter.copy_bytearray(data, (nbytes - 8), 8)

				self.mDataFull = data
				self.mEnabled = True
				result = True
			else:
				pass

		except Exception as ex:
			self.mLastException = ex
			raise(ex)

		return result

	##############################################################
	def getFullText(self):
		return self.mDataFull

	##############################################################
	def getDataText(self):
		return self.mData

	##############################################################
	def isAcknowledgeCommand(self):
		result = True if (self.mFlag & constant.RBCP_FLAG_ACKNOWLEDGE) != 0 else False
		return result

	##############################################################
	def isRequestCommand(self):
		result = not bool(self.isAcknowledgeCommand())
		return result

	##############################################################
	def isErrorOn(self):
		result = True if (self.mFlag & constant.RBCP_FLAG_ERR_BUSERROR) != 0 else False
		return result

	##############################################################
	def isEffectiveMessage(self):
		return self.mEnabled

	##############################################################
	def isExpectAck(self, req):

		result = True

		try:
			x = []
			x.append(req.mAddress == self.mAddress)
			x.append(req.mID == self.mID)
			x.append(req.mLength == self.mLength)
			x.append(req.mLength == len(self.mData))
			x.append(self.isAcknowledgeCommand())
			x.append(not self.isErrorOn())
			
			for i in range(len(x)):
				if x[i] != True:
					result = False
					break

		except Exception as ex:
			self.mLastException = ex
			result = False

		return result
	
	##############################################################
	@staticmethod
	def getAcknowledgeBytes(nlength):
		return	(nlength + 8)

